@@ -92,9 +92,11 @@ Playlist_remove_tracks(Playlist * self, PyObject *args)
         }
         Py_DECREF(item);
     }
+
     Py_BEGIN_ALLOW_THREADS;
     err = sp_playlist_remove_tracks(self->_playlist, tracks, num_tracks);
     Py_END_ALLOW_THREADS;
+
     return handle_error(err);
 }
 
@@ -670,7 +672,13 @@ playlist_image_changed_callback(sp_playlist * playlist, const byte * image,
     tramp = (Callback *) userdata;
     PyObject *p = Playlist_FromSpotify(playlist);
 
-    pimage = PyBytes_FromStringAndSize((const char *)image, 20);        //TODO: return Image
+    if (image) {
+        pimage = PyBytes_FromStringAndSize((const char *)image, 20);        //TODO: return Image
+    }
+    else {
+        Py_INCREF(Py_None);
+        pimage = Py_None;
+    }
     res = PyObject_CallFunctionObjArgs(tramp->callback,
                                        p, pimage, tramp->userdata, NULL);
     if (!res)
@@ -772,8 +780,7 @@ Playlist_rename(Playlist * self, PyObject *args)
 static PyObject *
 Playlist_owner(Playlist * self)
 {
-    PyErr_SetString(PyExc_NotImplementedError, "");
-    return NULL;
+    return User_FromSpotify(sp_playlist_owner(self->_playlist));
 }
 
 static PyObject *
