@@ -1,5 +1,6 @@
 #include <Python.h>
-#include <pyspotify.h>
+#include <libspotify/api.h>
+#include "pyspotify.h"
 
 Callback *
 create_trampoline(PyObject *callback, PyObject *manager, PyObject *userdata)
@@ -32,15 +33,21 @@ delete_trampoline(Callback * tr)
 PyObject *
 as_function(PyObject *o)
 {
-    if (PyFunction_Check(o)) {
-        return o;
-    }
-    else if (PyMethod_Check(o)) {
+    if (PyMethod_Check(o)) {
         return PyMethod_GET_FUNCTION(o);
     }
     else {
-        PyErr_SetString(SpotifyError,
-                        "Expected function or method for a callback.");
-        return NULL;
+        return o;
+    }
+}
+
+PyObject *
+error_message(int err)
+{
+    if (err != 0) {
+        return PyUnicode_FromString(sp_error_message(err));
+    }
+    else {
+        Py_RETURN_NONE;
     }
 }
