@@ -21,26 +21,6 @@
 static int session_constructed = 0;
 sp_session *g_session;
 
-static PyObject *
-PyTuple_NewByPreappending(PyObject *firstObject, PyObject *tuple)
-{
-    PyObject *result = PyTuple_New(PyObject_Length(tuple) + 1);
-
-    PyTuple_SetItem(result, 0, firstObject);
-    Py_XINCREF(firstObject);
-
-    unsigned i;
-
-    for (i = 0; i < PyObject_Length(tuple); ++i) {
-        PyObject *member = PyTuple_GetItem(tuple, i);
-
-        Py_XINCREF(member);
-        PyTuple_SetItem(result, i + 1, member);
-    }
-
-    return result;
-}
-
 static PyObject *
 Session_new(PyTypeObject * type, PyObject *args, PyObject *kwds)
 {
@@ -51,6 +31,15 @@ Session_new(PyTypeObject * type, PyObject *args, PyObject *kwds)
     return (PyObject *)self;
 }
 
+PyObject *
+Session_FromSpotify(sp_session * session)
+{
+    PyObject *s = PyObject_CallObject((PyObject *)&SessionType, NULL);
+
+    ((Session *) s)->_session = session;
+    return s;
+}
+
 static PyMemberDef Session_members[] = {
     {NULL}
 };
@@ -129,8 +118,8 @@ Session_playlist_container(Session * self)
 
     Py_BEGIN_ALLOW_THREADS;
     pc = sp_session_playlistcontainer(self->_session);
-
     Py_END_ALLOW_THREADS;
+
     PlaylistContainer *ppc =
         (PlaylistContainer *) PyObject_CallObject((PyObject *)
                                                   &PlaylistContainerType,
@@ -154,9 +143,11 @@ Session_load(Session * self, PyObject *args)
     }
     t = track->_track;
     s = self->_session;
+
     Py_BEGIN_ALLOW_THREADS;
     err = sp_session_player_load(s, t);
     Py_END_ALLOW_THREADS;
+
     return handle_error(err);
 }
 
@@ -167,9 +158,11 @@ Session_seek(Session * self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "i", &seek))
         return NULL;
+
     Py_BEGIN_ALLOW_THREADS;
     sp_session_player_seek(self->_session, seek);
     Py_END_ALLOW_THREADS;
+
     Py_RETURN_NONE;
 }
 
@@ -180,9 +173,11 @@ Session_play(Session * self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "i", &play))
         return NULL;
+
     Py_BEGIN_ALLOW_THREADS;
     sp_session_player_play(self->_session, play);
     Py_END_ALLOW_THREADS;
+
     Py_RETURN_NONE;
 }
 
@@ -201,6 +196,7 @@ Session_process_events(Session * self)
     Py_BEGIN_ALLOW_THREADS;
     sp_session_process_events(self->_session, &timeout);
     Py_END_ALLOW_THREADS;
+
     return Py_BuildValue("i", timeout);
 }
 
@@ -229,33 +225,53 @@ Session_search(Session * self, PyObject *args, PyObject *kwds)
     PyObject *callback, *userdata = NULL;
     int track_offset = 0, track_count = 32,
         album_offset = 0, album_count = 32, artist_offset = 0, artist_count =
-        32;
+        32, playlist_offset = 0, playlist_count = 32;
     Callback *st;
     PyObject *results;
+    sp_search_type search_type = SP_SEARCH_STANDARD;
+    char *str_search_type = NULL;
 
     static char *kwlist[] = { "query", "callback",
         "track_offset", "track_count",
         "album_offset", "album_count",
         "artist_offset", "artist_count",
-        "userdata", NULL
+        "playlist_offset", "playlist_count",
+        "search_type", "userdata", NULL
     };
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "esO|iiiiiiO", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "esO|iiiiiiiisO", kwlist,
                                      ENCODING, &query, &callback,
                                      &track_offset, &track_count,
                                      &album_offset, &album_count,
-                                     &artist_offset, &artist_count, &userdata))
+                                     &artist_offset, &artist_count,
+                                     &playlist_offset, &playlist_count,
+                                     &str_search_type, &userdata))
         return NULL;
+    /* Determine search type */
+    if (str_search_type) {
+        if (strcmp(str_search_type, "standard") == 0) {} // default
+        else if (strcmp(str_search_type, "suggest") == 0) {
+            search_type = SP_SEARCH_SUGGEST;
+        }
+        else {
+            PyErr_Format(SpotifyError, "Unknown search type: %s", str_search_type);
+            return NULL;
+        }
+    }
     if (!userdata)
         userdata = Py_None;
     st = create_trampoline(callback, NULL, userdata);
+
     Py_BEGIN_ALLOW_THREADS;
     search = sp_search_create(self->_session, query,
                               track_offset, track_count,
                               album_offset, album_count,
                               artist_offset, artist_count,
+                              playlist_offset, playlist_count,
+                              search_type,
                               (search_complete_cb *) search_complete,
                               (void *)st);
     Py_END_ALLOW_THREADS;
+
     results = Results_FromSpotify(search);
     return results;
 }
@@ -263,34 +279,18 @@ Session_search(Session * self, PyObject *args, PyObject *kwds)
 static PyObject *
 Session_browse_album(Session * self, PyObject *args, PyObject *kwds)
 {
-    PyObject *album, *callback, *userdata = NULL;
-    static char *kwlist[] = { "artist", "callback", "userdata", NULL };
-    if (!PyArg_ParseTupleAndKeywords
-        (args, kwds, "O!O|O", kwlist, &AlbumType, &album, &callback,
-         &userdata))
-        return NULL;
-
-    args = PyTuple_NewByPreappending((PyObject *)self, args);
+    /* Deprecated, calls the AlbumBrowserType object */
     PyObject *result =
         PyObject_Call((PyObject *)&AlbumBrowserType, args, kwds);
-    Py_XDECREF(args);
     return result;
 }
 
 static PyObject *
 Session_browse_artist(Session * self, PyObject *args, PyObject *kwds)
 {
-    PyObject *artist, *callback, *userdata = NULL;
-    static char *kwlist[] = { "artist", "callback", "userdata", NULL };
-    if (!PyArg_ParseTupleAndKeywords
-        (args, kwds, "O!O|O", kwlist, &ArtistType, &artist, &callback,
-         &userdata))
-        return NULL;
-
-    args = PyTuple_NewByPreappending((PyObject *)self, args);
+    /* Deprecated, calls the ArtistBrowserType object */
     PyObject *result =
         PyObject_Call((PyObject *)&ArtistBrowserType, args, kwds);
-    Py_XDECREF(args);
     return result;
 }
 
@@ -318,9 +318,11 @@ Session_set_preferred_bitrate(Session * self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "i", &bitrate))
         return NULL;
+
     Py_BEGIN_ALLOW_THREADS;
     sp_session_preferred_bitrate(self->_session, bitrate);
     Py_END_ALLOW_THREADS;
+
     Py_RETURN_NONE;
 }
 
@@ -332,11 +334,22 @@ Session_starred(Session * self)
     Py_BEGIN_ALLOW_THREADS;
     spl = sp_session_starred_create(self->_session);
     Py_END_ALLOW_THREADS;
+
     PyObject *pl = Playlist_FromSpotify(spl);
 
     return pl;
 }
 
+static PyObject *
+Session_flush_caches(Session * self)
+{
+    Py_BEGIN_ALLOW_THREADS;
+    sp_session_flush_caches(self->_session);
+    Py_END_ALLOW_THREADS;
+
+    Py_RETURN_NONE;
+}
+
 static PyMethodDef Session_methods[] = {
     {"username", (PyCFunction)Session_username, METH_NOARGS,
      "Return the canonical username for the logged in user"},
@@ -374,6 +387,8 @@ static PyMethodDef Session_methods[] = {
      "Set the preferred bitrate of the audio stream. 0 = 160k, 1 = 320k"},
     {"starred", (PyCFunction)Session_starred, METH_NOARGS,
      "Get the starred playlist for the logged in user"},
+    {"flush_caches", (PyCFunction)Session_flush_caches, METH_NOARGS,
+     "Flush the libspotify caches"},
     {NULL}
 };
 
@@ -571,7 +586,7 @@ notify_main_thread(sp_session * session)
     PyObject *client = (PyObject *)sp_session_userdata(session);
 
     if (client != NULL) {
-        method = PyObject_GetAttrString(client, "wake");
+        method = PyObject_GetAttrString(client, "notify_main_thread");
         res = PyObject_CallFunctionObjArgs(method, psession, NULL);
         if (!res)
             PyErr_WriteUnraisable(method);
@@ -619,7 +634,7 @@ music_delivery(sp_session * session, const sp_audioformat * format,
     int consumed = num_frames;  // assume all consumed
     if (!res)
         PyErr_WriteUnraisable(method);
-    if (PyInt_Check(res))
+    else if (PyInt_Check(res))
         consumed = (int)PyInt_AsLong(res);
     else if (PyLong_Check(res))
         consumed = (int)PyLong_AsLong(res);
@@ -713,6 +728,31 @@ end_of_track(sp_session * session)
     PyGILState_Release(gstate);
 }
 
+static void
+credentials_blob_updated(sp_session *session, const char *blob)
+{
+    PyGILState_STATE gstate;
+    PyObject *res, *method, *client, *py_session, *py_blob;
+
+#ifdef DEBUG
+        fprintf(stderr, "[DEBUG]-session- >> credentials_blob_updated called\n");
+#endif
+    gstate = PyGILState_Ensure();
+    client = (PyObject *)sp_session_userdata(session);
+    py_session = Session_FromSpotify(session);
+    py_blob = PyBytes_FromString(blob);
+
+    method = PyObject_GetAttrString(client, "credentials_blob_updated");
+    res = PyObject_CallFunctionObjArgs(method, py_session, py_blob, NULL);
+    if (!res)
+        PyErr_WriteUnraisable(method);
+    Py_DECREF(py_session);
+    Py_DECREF(py_blob);
+    Py_XDECREF(res);
+    Py_DECREF(method);
+    PyGILState_Release(gstate);
+}
+
 void
 session_init(PyObject *m)
 {
@@ -763,7 +803,15 @@ static sp_session_callbacks g_callbacks = {
     &music_delivery,
     &play_token_lost,
     &log_message,
-    &end_of_track
+    &end_of_track,
+    NULL, /* streaming_error */
+    NULL, /* userinfo_updated */
+    NULL, /* start_playback */
+    NULL, /* stop_playback */
+    NULL, /* get_audio_buffer_stats */
+    NULL, /* offline_status_updated */
+    NULL, /* offline_error */
+    &credentials_blob_updated,
 };
 
 PyObject *
@@ -776,6 +824,7 @@ session_connect(PyObject *self, PyObject *args)
     char *username, *password;
     char *cache_location, *settings_location, *user_agent;
     bool relogin = 0, remember_me;
+    char *blob = NULL;
 
     if (!PyArg_ParseTuple(args, "O", &client))
         return NULL;
@@ -851,12 +900,30 @@ session_connect(PyObject *self, PyObject *args)
     PyErr_Clear();
     Py_XDECREF(remember);
 
+    /* Binary blob to use in place of a password */
+    PyObject *py_blob = PyObject_GetAttr(client, PyBytes_FromString("login_blob"));
+    if (py_blob != NULL) {
+        if (!(PyBytes_Check(py_blob))) {
+            PyErr_SetString(SpotifyError, "login_blob must be a string of bytes.");
+            Py_XDECREF(py_blob);
+            return NULL;
+        }
+        blob = PyString_AS_STRING(py_blob);
+        /* if blob is the empty string (= default value), we pass NULL to
+         * sp_session_login */
+        if (blob[0] == '\0')
+            blob = NULL;
+    } else {
+        PyErr_Clear();
+    }
+
 #ifdef DEBUG
     fprintf(stderr, "[DEBUG]-session- creating session...\n");
 #endif
     error = sp_session_create(&config, &session);
     if (error != SP_ERROR_OK) {
         PyErr_SetString(SpotifyError, sp_error_message(error));
+        Py_XDECREF(py_blob);
         return NULL;
     }
     session_constructed = 1;
@@ -868,6 +935,7 @@ session_connect(PyObject *self, PyObject *args)
         error = sp_session_relogin(session);
         if (error != SP_ERROR_OK) {
             PyErr_SetString(SpotifyError, sp_error_message(error));
+            Py_XDECREF(py_blob);
             return NULL;
         }
     } else {
@@ -875,8 +943,11 @@ session_connect(PyObject *self, PyObject *args)
     fprintf(stderr, "[DEBUG]-session- login as %s in progress...\n",
             username);
 #endif
-        sp_session_login(session, username, password, remember_me);
+        Py_BEGIN_ALLOW_THREADS;
+        sp_session_login(session, username, password, remember_me, blob);
+        Py_END_ALLOW_THREADS;
     }
+    Py_XDECREF(py_blob);
     g_session = session;
     Session *psession =
         (Session *) PyObject_CallObject((PyObject *)&SessionType, NULL);
