@@ -11,3 +11,6 @@ extern void session_init(PyObject *m);
 extern PyObject *session_connect(PyObject *self, PyObject *args);
 
 extern PyObject *handle_error(int error);
+
+PyObject *
+Session_FromSpotify(sp_session * session);
